#ifndef TEST_UTILS_CRYPT_MEMORYCRYPT_MEMORYENCRYPT_H
#define TEST_UTILS_CRYPT_MEMORYCRYPT_MEMORYENCRYPT_H

#include <check.h>

TCase* test_case_memoryEncrypt();

#endif  // TEST_UTILS_CRYPT_MEMORYCRYPT_MEMORYENCRYPT_H
