#ifndef TEST_UTILS_STRINGUTILS_OIDC_STRNCOPY_H
#define TEST_UTILS_STRINGUTILS_OIDC_STRNCOPY_H

#include <check.h>

TCase* test_case_oidc_strncopy();

#endif  // TEST_UTILS_STRINGUTILS_OIDC_STRNCOPY_H
