#ifndef TEST_UTILS_CRYPT_CRYPT_CRYPT_ENCRYPT_H
#define TEST_UTILS_CRYPT_CRYPT_CRYPT_ENCRYPT_H

#include <check.h>

TCase* test_case_crypt_encrypt();

#endif  // TEST_UTILS_CRYPT_CRYPT_CRYPT_ENCRYPT_H
