#ifndef OIDC_GEN_H
#define OIDC_GEN_H

#include "defines/version.h"
#include "oidc-gen_options.h"

const char* argp_program_version = GEN_VERSION;

const char* argp_program_bug_address = BUG_ADDRESS;

#endif  // OIDC_GEN_H
