#ifndef TEST_UTILS_STRINGUTILS_ESCAPECHARINSTR_H
#define TEST_UTILS_STRINGUTILS_ESCAPECHARINSTR_H

#include <check.h>

TCase* test_case_escapeCharInStr();

#endif  // TEST_UTILS_STRINGUTILS_ESCAPECHARINSTR_H
