#ifndef TEST_UTILS_PORTUTILS_FINDREDIRECTURIBYPORT_H
#define TEST_UTILS_PORTUTILS_FINDREDIRECTURIBYPORT_H

#include <check.h>

TCase* test_case_findRedirectUriByPort();

#endif  // TEST_UTILS_PORTUTILS_FINDREDIRECTURIBYPORT_H
