#ifndef TEST_UTILS_URIUTILS_EXTRACTPARAMETERVALUEFROMURI_H
#define TEST_UTILS_URIUTILS_EXTRACTPARAMETERVALUEFROMURI_H

#include <check.h>

TCase* test_case_extractParameterValueFromUri();

#endif  // TEST_UTILS_URIUTILS_EXTRACTPARAMETERVALUEFROMURI_H
