#ifndef TEST_UTILS_CRYPT_CRYPT_FROMBASE64URLSAFE_H
#define TEST_UTILS_CRYPT_CRYPT_FROMBASE64URLSAFE_H

#include <check.h>

TCase* test_case_fromBase64UrlSafe();

#endif  // TEST_UTILS_CRYPT_CRYPT_FROMBASE64URLSAFE_H
