#ifndef TEST_UTILS_STRINGUTILS_STRELIMIFFOLLOWED_H
#define TEST_UTILS_STRINGUTILS_STRELIMIFFOLLOWED_H

#include <check.h>

TCase* test_case_strelimIfFollowed();

#endif  // TEST_UTILS_STRINGUTILS_STRELIMIFFOLLOWED_H
