#ifndef TEST_UTILS_CRYPT_CRYPT_TOBASE64URLSAFE_H
#define TEST_UTILS_CRYPT_CRYPT_TOBASE64URLSAFE_H

#include <check.h>

TCase* test_case_toBase64UrlSafe();

#endif  // TEST_UTILS_CRYPT_CRYPT_TOBASE64URLSAFE_H
