#ifndef TEST_UTILS_CRYPT_MEMORYCRYPT_MEMORYDECRYPT_H
#define TEST_UTILS_CRYPT_MEMORYCRYPT_MEMORYDECRYPT_H

#include <check.h>

TCase* test_case_memoryDecrypt();

#endif  // TEST_UTILS_CRYPT_MEMORYCRYPT_MEMORYDECRYPT_H
