#ifndef OIDC_ADD_H
#define OIDC_ADD_H

#include "defines/version.h"
#include "oidc-add_options.h"

const char* argp_program_version = ADD_VERSION;

const char* argp_program_bug_address = BUG_ADDRESS;

#endif  // OIDC_ADD_H
