#ifndef COMMON_FEATURES_H
#define COMMON_FEATURES_H

void common_handleListConfiguredAccountConfigs();
void common_assertAgent(unsigned char remote);
void common_assertOidcPrompt();

#endif  // COMMON_FEATURES_H
